package com.sxc.etaxincome.domain.business.impl;

import com.alibaba.nacos.shaded.com.google.common.collect.Lists;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.google.common.collect.Maps;
import com.sxc.etaxincome.common.model.report.IncomeExceptionStatResult;
import com.sxc.etaxincome.criteria.IncomeQueryCommon;
import com.sxc.etaxincome.domain.business.IExceptionManager;
import com.sxc.etaxincome.domain.constants.EtaxExceptionConstants;
import com.sxc.etaxincome.domain.model.report.AnalysisCriteria;
import com.sxc.etaxincome.domain.model.report.ExceptionStat;
import com.sxc.etaxincome.domain.support.DateProvider;
import com.sxc.etaxincome.service.IFinRealLogService;
import com.sxc.framework.common.date.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ExceptionManager implements IExceptionManager {

    @Autowired
    private IFinRealLogService finRealLogService;

    @Override
    public List<ExceptionStat> getExceptionStatList(AnalysisCriteria analysisCriteria) {
        IncomeQueryCommon incomeQueryCommon = IncomeQueryCommon.builder()
                .userName(analysisCriteria.getUsername())
                .areaCode(analysisCriteria.getAreaCode())
                .startDateTime(LocalDateTime.parse(DateProvider.getLongDateTimeHead(analysisCriteria.getStartDateTime()), DateTimeFormatter.ofPattern(DateUtils.YMDHMS_DASH)))
                .endDateTime(LocalDateTime.parse(DateProvider.getLongDateTimeTail(analysisCriteria.getEndDateTime()), DateTimeFormatter.ofPattern(DateUtils.YMDHMS_DASH)))
                .build();
        List<IncomeExceptionStatResult> incomeInvoiceApplyList = finRealLogService.queryExceptionList(incomeQueryCommon);
        if (CollectionUtils.isEmpty(incomeInvoiceApplyList)) {
            return Collections.emptyList();
        }

        Map<String, Integer> exceptionMap = Maps.newLinkedHashMapWithExpectedSize(10);
        incomeInvoiceApplyList.forEach(e -> {
            if(StringUtils.isEmpty(e.getResMsg())) {
                e.setResMsg(EtaxExceptionConstants.ETAX_EXCEPTION_TAX_NULL);
            }
            if(e.getResMsg().contains(EtaxExceptionConstants.ETAX_EXCEPTION_TAX_BUSY)) {
                e.setResMsg(EtaxExceptionConstants.ETAX_EXCEPTION_TAX_BUSY);
            }
            if(null==exceptionMap.get(e.getResMsg())) {
                exceptionMap.put(e.getResMsg(), e.getResNum());
            } else {
                exceptionMap.put(e.getResMsg(), exceptionMap.get(e.getResMsg()) + e.getResNum());
            }
        });

        List<ExceptionStat> exceptionStatList = Lists.newArrayListWithExpectedSize(10);
        exceptionMap.forEach((key, value) -> exceptionStatList.add(ExceptionStat.builder().message(key).count(value).build()));

        List<ExceptionStat> exceptionSortedList = exceptionStatList.stream().sorted(Comparator.comparing(ExceptionStat::getCount).reversed()).limit(8).collect(Collectors.toList());
        return exceptionSortedList;
    }

}
