package com.sxc.etaxincome.domain.command.analysis;

import com.sxc.etaxincome.common.enums.CountTypeCode;
import com.sxc.etaxincome.common.model.report.IncomeInvoiceStatResult;
import com.sxc.etaxincome.criteria.IncomeQueryCommon;
import com.sxc.etaxincome.domain.model.report.AnalysisCriteria;
import com.sxc.etaxincome.domain.support.DateProvider;
import com.sxc.etaxincome.service.IFinCountReportService;
import com.sxc.framework.common.date.DateUtils;
import com.sxc.framework.common.json.JSONUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

@Component(value = "analysisCommand")
public class AnalysisCommand implements IAnalysisCommand {

    @Autowired
    private IFinCountReportService finCountReportService;

    @Override
    public String handle(AnalysisCriteria analysisCriteria) {
        IncomeQueryCommon incomeQueryCommon = IncomeQueryCommon.builder()
                .userName(analysisCriteria.getUsername())
                .countType(analysisCriteria.getCountType())
                .startDateTime(LocalDateTime.parse(DateProvider.getLongDateTimeHead(analysisCriteria.getStartDateTime()), DateTimeFormatter.ofPattern(DateUtils.YMDHMS_DASH)))
                .endDateTime(LocalDateTime.parse(DateProvider.getLongDateTimeTail(analysisCriteria.getEndDateTime()), DateTimeFormatter.ofPattern(DateUtils.YMDHMS_DASH)))
                .build();
        List<IncomeInvoiceStatResult> incomeInvoiceStatResultList = finCountReportService.queryReportListByDate(incomeQueryCommon);
        incomeInvoiceStatResultList.stream().forEach(e -> {
            if(!analysisCriteria.getCountType().equals(CountTypeCode.HOUR.getCode())) {
                e.setStatDate(e.getStatDate().substring(0, 10));
            }
        });
        return JSONUtil.toString(incomeInvoiceStatResultList);
    }

}
