package com.sxc.etaxincome.domain.command.analysis;

import cn.hutool.core.date.DateUtil;
import com.google.common.collect.Lists;
import com.sxc.etaxincome.criteria.IncomeQueryCommon;
import com.sxc.etaxincome.domain.model.report.AnalysisCriteria;
import com.sxc.etaxincome.domain.model.report.SellerGrowth;
import com.sxc.etaxincome.domain.support.DateProvider;
import com.sxc.etaxincome.entity.FinUserTaxno;
import com.sxc.etaxincome.service.IFinUserTaxnoService;
import com.sxc.framework.common.json.JSONUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author caosongqing
 * @date 2023/11/10
 * @description
 */
@Component(value = "sellerGrowthCommand")
public class SellerGrowthCommand implements IAnalysisCommand {

    @Autowired
    private IFinUserTaxnoService finUserTaxnoService;

    @Override
    public String handle(AnalysisCriteria analysisCriteria) {
        String startDate = analysisCriteria.getStartDateTime();
        String endDate = analysisCriteria.getEndDateTime();
        List<String> dateList = DateProvider.dateListBetween(startDate, endDate);
        List<SellerGrowth> sellerGrowthList = Lists.newArrayListWithExpectedSize(20);
        for (String targetDate : dateList) {
            IncomeQueryCommon incomeQueryCommon = IncomeQueryCommon.builder()
                    .userName(analysisCriteria.getUsername())
                    .startDateTime(DateUtil.parseLocalDateTime(DateProvider.getDateTimeHead(targetDate)))
                    .endDateTime(DateUtil.parseLocalDateTime(DateProvider.getDateTimeTail(targetDate))).build();
            List<FinUserTaxno> makeoutInvoiceSellerList = finUserTaxnoService.queryList(incomeQueryCommon);
            sellerGrowthList.add(SellerGrowth.builder().date(targetDate).count(makeoutInvoiceSellerList.size()).build());
        }
        return JSONUtil.toString(sellerGrowthList);
    }
}
