package com.sxc.etaxincome.domain.command.report;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.api.config.annotation.NacosValue;
import com.google.common.base.Stopwatch;
import com.sxc.etaxincome.common.util.RetryHttpHelper;
import com.sxc.etaxincome.domain.business.IApiConfigManager;
import com.sxc.etaxincome.domain.exception.EtaxIncomeException;
import com.sxc.etaxincome.domain.model.common.CommonRequestWrapper;
import com.sxc.etaxincome.domain.model.common.CommonResponse;
import com.sxc.etaxincome.domain.model.cookie.TokenNecessary;
import com.sxc.etaxincome.domain.model.report.ApiConfig;
import com.sxc.etaxincome.domain.model.report.ForceReportRequestWrapper;
import com.sxc.etaxincome.domain.model.report.IReportResponse;
import com.sxc.etaxincome.domain.template.ResponseHandlerTemplate;
import com.sxc.etaxincome.enums.ApiCode;
import com.sxc.framework.common.httpclient.ResponseContent;
import com.sxc.framework.common.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.TimeUnit;

import static com.sxc.etaxincome.domain.constants.EtaxReportConstants.*;

@Slf4j
@Component(value = "forceReportCommand")
public class ForceReportCommand implements IReportCommand {

    @NacosValue(value = "${url.etax.report}", autoRefreshed = true)
    private String etaxReportUrl;

    @Autowired
    private IApiConfigManager apiConfigManager;

    @Override
    public CommonResponse<IReportResponse> execute(CommonRequestWrapper commonRequestWrapper) {
        Stopwatch stopwatch = Stopwatch.createStarted();
        ApiConfig apiConfig = apiConfigManager.getApiConfig(ApiCode.FORCE_REPORT.getCode());
        JSONObject requestDataJsonObject = new JSONObject();
        String requestDataStr = commonRequestWrapper.getData();
        String requestSerialNo = commonRequestWrapper.getRequestSerialNo();
        log.info("cancel report requestSerialNo:{} data:{}", requestSerialNo, requestDataStr);
        ForceReportRequestWrapper forceReportRequestWrapper = JSON.parseObject(requestDataStr, ForceReportRequestWrapper.class);
        TokenNecessary tokenNecessary = forceReportRequestWrapper.decodeToken();
        String token = tokenNecessary.getToken();
        requestDataJsonObject.put(API_PARAM_TOKEN, token);
        requestDataJsonObject.put(API_PARAM_AREA_CODE, forceReportRequestWrapper.getArea());
        String test = forceReportRequestWrapper.getTest();
        if (StringUtils.isNotBlank(test)) {
            requestDataJsonObject.put(API_PARAM_TEST, test);
        }
        if (StringUtils.isNotBlank(forceReportRequestWrapper.getPch())) {
            requestDataJsonObject.put("pch", forceReportRequestWrapper.getPch());
        }
        if (StringUtils.isNotBlank(forceReportRequestWrapper.getReportType())) {
            requestDataJsonObject.put("yzpzzlDm", forceReportRequestWrapper.getReportType());
        }
        if (StringUtils.isNotBlank(forceReportRequestWrapper.getSbxh())) {
            requestDataJsonObject.put("sbxh", forceReportRequestWrapper.getSbxh());
        }
        if (StringUtils.isNotBlank(forceReportRequestWrapper.getForceReason())) {
            requestDataJsonObject.put("qtyy", forceReportRequestWrapper.getForceReason());
        }
        try {
            String etaxReportUrlFull = etaxReportUrl + apiConfig.getApiUrl();
            log.info("url:{}", etaxReportUrlFull);
            String requestJson = JSONUtil.toString(requestDataJsonObject);
            log.info("request:{}", requestJson);
            ResponseContent responseContent = RetryHttpHelper.retryPostJsonEntity(etaxReportUrlFull, requestJson);
            String response = responseContent.getContent();
            log.info("response:{}", response);
            stopwatch.stop();
            log.info("cancel report elapsed time:{}ms", stopwatch.elapsed(TimeUnit.MILLISECONDS));
            return new ResponseHandlerTemplate<IReportResponse>() {
                @Override
                public CommonResponse<IReportResponse> handleSuccess(JSONObject jsonObject) {
                    String message = jsonObject.getString(ETAX_RESPONSE_MESSAGE_KEY);
                    return CommonResponse.success(message);
                }
            }.handle(response);
        } catch (Exception e) {
            log.error("cancel report error:", e);
            throw new EtaxIncomeException(e);
        }
    }
}
