package com.sxc.etaxincome.domain.model.common;

import com.alibaba.fastjson2.JSON;
import com.sxc.etaxincome.domain.exception.EtaxIncomeException;
import com.sxc.etaxincome.domain.model.cookie.CookieCarrier;
import com.sxc.etaxincome.domain.model.cookie.TokenNecessary;
import com.sxc.framework.common.code.EncryptUtil;
import com.sxc.framework.common.json.JSONUtil;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;

import static com.sxc.etaxincome.domain.constants.EtaxReportConstants.PROXY_KEY_IN_COOKIE;

/**
 * @author caosongqing
 * @date 2022/9/21
 * @description
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@SuperBuilder
public class BusinessCommonRequestWrapper implements Serializable {
    private static final long serialVersionUID = -4090578281523166142L;
    private String taxno;
    private String area;
    private String site;
    private String token;
    private String test;

    public String getPlainToken() {
        return this.decodeToken().getToken();
    }

    public TokenNecessary decodeToken() {
        String decodeToken = token;
        if (token.startsWith(EncryptUtil.FKEY)) {
            decodeToken = EncryptUtil.decryptAesCk(StringUtils.replace(token, " ", "+"));
        }
        String targetToken;
        String proxy;
        String session = null;
        if (JSONUtil.isJSONObject(decodeToken)) {
            CookieCarrier cookieCarrier = JSON.parseObject(decodeToken, CookieCarrier.class);
            targetToken = cookieCarrier.getCookie();
            proxy = cookieCarrier.getProxyIp();
            session = cookieCarrier.getSession();
        } else {
            if (decodeToken.length() < 100) {
                throw new IllegalArgumentException("Token不合法，请检查");
            }
            targetToken = delCookieProxy(decodeToken);
            proxy = getCookieProxy(decodeToken);
        }
        return TokenNecessary.builder().token(targetToken).proxy(proxy).session(session).build();
    }

    public String getCookieProxy(String cookie) {
        if (cookie != null && cookie.contains(PROXY_KEY_IN_COOKIE)) {
            String[] ss = cookie.split(";");
            for (int i = 0; i < ss.length; i++) {
                if (ss[i].contains(PROXY_KEY_IN_COOKIE)) {
                    String[] ssss = ss[i].split("=");
                    return ssss[1];
                }
            }
        }
        return "";
    }

    public String delCookieProxy(String cookie) {
        if (cookie != null && cookie.contains(PROXY_KEY_IN_COOKIE)) {
            String[] ss = cookie.split(";");
            for (int i = 0; i < ss.length; i++) {
                if (ss[i].contains(PROXY_KEY_IN_COOKIE)) {
                    return cookie.replace(ss[i], "");
                }
            }
        }
        return cookie;
    }

    public void doCheckRequestBusinessParam() {
        if (StringUtils.isBlank(this.getArea())) {
            throw new EtaxIncomeException("area is empty");
        } else if (this.getArea().length() != 4) {
            throw new EtaxIncomeException("area is invalid");
        }
        if (StringUtils.isBlank(this.getTaxno())) {
            throw new EtaxIncomeException("taxno is empty");
        }
        if (StringUtils.isBlank(this.getToken())) {
            throw new EtaxIncomeException("token is empty");
        }
    }
}
