package com.sxc.etaxincome.domain.model.common;

import com.alibaba.fastjson2.annotation.JSONField;
import com.google.common.collect.Lists;
import com.sxc.etaxincome.enums.ResponseCode;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.List;

/**
 * @author caosongqing
 * @date 2024/4/23
 * @description
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
public class CommonResponse<T> implements Serializable {

    private static final long serialVersionUID = 3212054091102827090L;
    /**
     * code
     */
    @JSONField(name = "resultCode")
    private String resultCode;
    /**
     * message
     */
    @JSONField(name = "resultMessage")
    private String resultMessage;
    /**
     * data
     */
    @JSONField(name = "data")
    @SuppressWarnings("java:S1948")
    private T data;

    private CommonResponse(String resultCode, String resultMessage) {
        this.resultCode = resultCode;
        this.resultMessage = resultMessage;
        this.data = null;
    }

    public boolean resultSuccess() {
        return ResponseCode.SUCCESS.getCode().equals(resultCode);
    }

    public boolean invalidToken() {
        return ResponseCode.TOKEN_INVALID.getCode().equals(resultCode);
    }

    public boolean requiredQrcodeSecurity() {
        return ResponseCode.ETAX_QR_CODE_CHECK.getCode().equals(resultCode);
    }

    public boolean notifyIgnore() {
        List<ResponseCode> ignoreResponseCodeList = Lists.newArrayListWithExpectedSize(10);
        ignoreResponseCodeList.add(ResponseCode.SUCCESS);
        ignoreResponseCodeList.add(ResponseCode.ETAX_LOGIN_INVALID);
        ignoreResponseCodeList.add(ResponseCode.ETAX_QR_CODE_CHECK);
        ignoreResponseCodeList.add(ResponseCode.TOKEN_INVALID);
        ignoreResponseCodeList.add(ResponseCode.MAKEOUT_INVOICE_NO_EXPIRED);
        ignoreResponseCodeList.add(ResponseCode.MAKEOUT_INVOICE_NO_REQUIRED);
        ignoreResponseCodeList.add(ResponseCode.ERROR_TAX);
        ignoreResponseCodeList.add(ResponseCode.MAKEOUT_INVOICE_PROXY_ERROR);
        ignoreResponseCodeList.add(ResponseCode.NOT_FOUND_ITEM);
        ignoreResponseCodeList.add(ResponseCode.OVER_CREDIT_LINE);
        ignoreResponseCodeList.add(ResponseCode.FREEZE_CREDIT_LINE);
        ignoreResponseCodeList.add(ResponseCode.DISCOUNT_OVER_ITEM_AMOUNT);
        ignoreResponseCodeList.add(ResponseCode.WRONG_FORMAT_BANK_ACCOUNT);
        ignoreResponseCodeList.add(ResponseCode.ILLEGAL_PARAM);
        ignoreResponseCodeList.add(ResponseCode.RESPONSE_419_ERROR);
        ignoreResponseCodeList.add(ResponseCode.NOT_FOUND_BUYER_TAXNO);
        return ignoreResponseCodeList.contains(ResponseCode.as(resultCode));
    }

    public static <T> CommonResponse<T> success(T data) {
        return new CommonResponse<>(ResponseCode.SUCCESS.getCode(), ResponseCode.SUCCESS.getDesc(), data);
    }

    public static <T> CommonResponse<T> success() {
        return success(null);
    }

    public static <T> CommonResponse<T> success(String message) {
        return new CommonResponse<>(ResponseCode.SUCCESS.getCode(), message);
    }

    public static <T> CommonResponse<T> error(ResponseCode responseCode) {
        return new CommonResponse<>(responseCode.getCode(), responseCode.getDesc());
    }

    public static <T> CommonResponse<T> error(ResponseCode responseCode, String msg) {
        return new CommonResponse<>(responseCode.getCode(), msg);
    }

    public static <T> CommonResponse<T> error(ResponseCode responseCode, T data) {
        return new CommonResponse<>(responseCode.getCode(), responseCode.getDesc(), data);
    }

    public static <T> CommonResponse<T> error(ResponseCode responseCode, String msg, T data) {
        return new CommonResponse<>(responseCode.getCode(), msg, data);
    }

    public static <T> CommonResponse<T> error(String code, String msg) {
        return new CommonResponse<>(code, msg);
    }

    public static <T> CommonResponse<T> error(String msg) {
        return new CommonResponse<>(ResponseCode.ERROR.getCode(), msg);
    }
}
