package com.sxc.etaxincome.enums;

import lombok.Getter;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * general exception code.
 * @author caosongqing
 * @date 2024-4-23
 */
@Getter
public enum ResponseCode {
    /**
     * success.
     */
    SUCCESS("00", "成功"),
    /**
     * user not found.
     */
    NO_USER_FOUND("01", "无此用户"),
    /**
     * sign wrong.
     */
    SIGN_WRONG("02", "签名错误"),
    /**
     * parameter required.
     */
    PARAMETER_REQUIRED("03", "参数不全"),
    /**
     * token invalid.
     */
    TOKEN_INVALID("04", "Token无效"),
    /**
     * service not found.
     */
    NO_SERVICE_FOUND("05", "无此服务"),
    /**
     * privilege not found.
     */
    NO_PRIVILEGE_FOUND("06", "没有权限"),
    /**
     * taxno not registered.
     */
    TAXNO_NOT_REGISTERED("07", "该税号未注册"),
    /**
     * taxcount exceed maximum
     */
    TAXCOUNT_EXCEED_MAX("08", "超过授权税号总数"),
    ACCOUNT_DISABLED("09", "账号已禁用"),
    NO_PRIVILEGE_AREA_FOUND("10", "没有地区权限"),
    PARAMETER_REQUIRED_USERNAME("11", "缺少用户名"),
    PARAMETER_REQUIRED_TAXNO("12", "缺少税号"),
    SMS_SEND("13", "已发送短信"),
    LOGIN_DENY("14", "无法登录"),
    LOGIN_WAITING("15", "登录遇阻，可重试"),
    PARAMETER_REQUIRED_AREA("16", "缺少地区"),
    DIFFERENT_MAKEOUT_INVOICE_API_CODE("22", "同一流水号不允许变更开票接口"),
    AUTH_PERMISSION_QUERY_FAIL("23", "查询功能权限失败"),
    AUTH_PERMISSION_NOT_FOUND("24", "功能未授权"),
    AUTH_USER_QUERY_FAIL("25", "查询用户失败"),
    AUTH_USER_NOT_FOUND("26", "未查询到用户"),
    AUTH_DATA_PRIVILEGE_QUERY_FAIL("27", "查询权限失败"),
    AUTH_DATA_PRIVILEGE_NOT_FOUND("28", "税号未授权"),
    DIFFERENT_MAKEOUT_INVOICE_TYPE("29", "同一流水号不允许变更开票类型"),
    DIFFERENT_MAKEOUT_INVOICE_MODE("30", "同一流水号不允许变更开票模式"),
    ETAX_LOGIN_INVALID("31", "请重新登录后继续办理业务"),
    ETAX_QR_CODE_CHECK("32", "需要扫描二维码进行身份校验"),
    NOT_FOUND_BUYER_TAXNO("33", "未查询到购买方纳税人信息，请维护"),
    ERROR_TAX("34", "税率不合法"),
    OVER_CREDIT_LINE("35", "授信额度不够"),
    NOT_FOUND_ITEM("36", "未查询到对应的项目信息"),
    DISCOUNT_OVER_ITEM_AMOUNT("37", "折扣金额大于商品金额"),
    WRONG_FORMAT_BANK_ACCOUNT("38", "销售方银行账号格式不正确"),
    FREEZE_CREDIT_LINE("39", "暂停授信"),
    READ_TIMED_OUT("40", "开票请求超时，请检查是否开票成功"),
    MAKEOUT_INVOICE_DOING_FROM_TAX_BUREAU("41", "发票正在开具中（税局正在处理中）"),

    ETAX_RESPONSE_ERROR("42", "税局处理异常，请检查是否开票成功"),
    ETAX_DOWN_ERROR("43", "税局暂不可用，请稍后重试"),
    MAKEOUT_INVOICE_APPLY_DETAIL_SAVE_FAIL("50", "开票请求详情保存失败"),

    MAKEOUT_INVOICE_APPLY_SAVE_FAIL("51", "开票请求保存失败"),
    /**
     * invoice made out.
     */
    MAKEOUT_INVOICE_DONE("52", "发票已经开具"),
    MAKEOUT_INVOICE_APPLY_UPDATE_FAIL("53", "开票请求更新失败"),
    /**
     * invoice make out fail
     */
    MAKEOUT_INVOICE_FAIL("54", "开票失败"),
    MAKEOUT_INVOICE_RESULT_SAVE_FAIL("55", "开票结果保存失败"),
    MAKEOUT_INVOICE_NO_EXPIRED("56", "当前开票信息已失效，请重试"),
    MAKEOUT_INVOICE_NO_RESULT("57", "发票已开具，但未查询到开票结果"),
    MAKEOUT_INVOICE_DOING("58", "开票正在进行中（开票服务正在处理中）"),
    MAKEOUT_INVOICE_DONE_GET_RESULT_MATCHED("59", "发票已经开具，通过查询最近开票列表匹配到开票结果"),

    MAKEOUT_INVOICE_NO_REQUIRED("60", "当前开票信息不完整，请重试"),

    MAKEOUT_INVOICE_DONE_GET_RESULT_FAIL("61", "发票已经开具，但未获得开票结果"),
    MAKEOUT_INVOICE_DONE_GET_RESULT_NOT_MATCHED("62", "发票已经开具，通过查询最近开票列表未匹配到开票结果"),

    MAKEOUT_INVOICE_SUCCESS_SUBMITTING_FAIL_GET_RESULT_EXPIRED_MAKEOUT_INVOICE_NO("63", "发票已经成功提交开具，但一直未获得开票结果，再次提交开票时，开票信息已失效，终止开票"),

    MAKEOUT_INVOICE_RED_QUERY_RESULT_FAIL("65", "开具红票录入成功，但查询开票结果失败"),
    MAKEOUT_INVOICE_PROXY_ERROR("70", "请求代理异常，请稍后重试"),
    RESPONSE_419_ERROR("71", "请求发生异常，请稍后重试"),
    ILLEGAL_PARAM("72", "参数校验不通过"),
    SIGN_STRENGTH_ERROR("73", "请求税局发生异常，请稍后重试"),
    CALL_PROXY_API_UNKNOWN_ERROR("74", SIGN_STRENGTH_ERROR.desc),
    GET_MAKEOUT_INVOICE_NO_ERROR("75", "获取开票号失败，请稍后重试"),
    MAKEOUT_INVOICE_FAIL_RETRY_LATER("76", SIGN_STRENGTH_ERROR.desc),
    UPGRADE_SIGN("80", "非法请求"),
    MAKEOUT_INVOICE_REVOKE("90", "已作废，不可重新开票"),
    /**
     * error.
     */
    ERROR("99", "网络繁忙，请稍后重试");

    private final String code;
    private final String desc;

    ResponseCode(String code, String desc) {
        this.code = code;
        this.desc = desc;
    }

    private static final Map<String, ResponseCode> ENUM_CACHE = new HashMap<>();

    static {
        for (ResponseCode responseCode : ResponseCode.values()) {
            ENUM_CACHE.put(responseCode.code, responseCode);
        }
    }

    public static ResponseCode as(String code) {
        for (ResponseCode responseCode : ResponseCode.values()) {
            if (responseCode.code.equals(code)) {
                return responseCode;
            }
        }
        return null;
    }

    public static String getDesc(String code) {
        return Optional.ofNullable(ENUM_CACHE.get(code))
                .map(ResponseCode::getDesc)
                .orElseThrow(() -> new IllegalArgumentException("invalid exception code."));
    }

}